@@ -4,12 +4,12 @@
  *	  support code for nodes (now that we have removed the home-brew
  *	  inheritance system, our support code for nodes is much simpler)
  *
- * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/nodes/nodes.c,v 1.30 2009/01/01 17:23:43 momjian Exp $
+ *	  src/backend/nodes/nodes.c
  *
  * HISTORY
  *	  Andrew Yu			Oct 20, 1994	file creation
