@@ -14,7 +14,7 @@
  *
  *	Initial author: Simon Riggs		simon@2ndquadrant.com
  *
- * Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -236,7 +236,7 @@ PgArchiverMain(int argc, char *argv[])
 	/*
 	 * Identify myself via ps
 	 */
-	init_ps_display("archiver process", "", "", "");
+	init_ps_display("archiver", "", "", "");
 
 	pgarch_MainLoop();
 
@@ -596,10 +596,17 @@ pgarch_archiveXlog(char *xlog)
 					 errhint("See C include file \"ntstatus.h\" for a description of the hexadecimal value."),
 					 errdetail("The failed archive command was: %s",
 							   xlogarchcmd)));
-#else
+#elif defined(HAVE_DECL_SYS_SIGLIST) && HAVE_DECL_SYS_SIGLIST
 			ereport(lev,
 					(errmsg("archive command was terminated by signal %d: %s",
-							WTERMSIG(rc), pg_strsignal(WTERMSIG(rc))),
+							WTERMSIG(rc),
+							WTERMSIG(rc) < NSIG ? sys_siglist[WTERMSIG(rc)] : "(unknown)"),
+					 errdetail("The failed archive command was: %s",
+							   xlogarchcmd)));
+#else
+			ereport(lev,
+					(errmsg("archive command was terminated by signal %d",
+							WTERMSIG(rc)),
 					 errdetail("The failed archive command was: %s",
 							   xlogarchcmd)));
 #endif
@@ -665,11 +672,6 @@ pgarch_readyXlog(char *xlog)
 
 	snprintf(XLogArchiveStatusDir, MAXPGPATH, XLOGDIR "/archive_status");
 	rldir = AllocateDir(XLogArchiveStatusDir);
-	if (rldir == NULL)
-		ereport(ERROR,
-				(errcode_for_file_access(),
-				 errmsg("could not open archive status directory \"%s\": %m",
-						XLogArchiveStatusDir)));
 
 	while ((rlde = ReadDir(rldir, XLogArchiveStatusDir)) != NULL)
 	{
