@@ -3,7 +3,7 @@
  * origin.c
  *	  Logical replication progress tracking support.
  *
- * Copyright (c) 2013-2017, PostgreSQL Global Development Group
+ * Copyright (c) 2013-2018, PostgreSQL Global Development Group
  *
  * IDENTIFICATION
  *	  src/backend/replication/logical/origin.c
@@ -60,7 +60,7 @@
  *	 all our platforms, but it also simplifies memory ordering concerns
  *	 between the remote and local lsn. We use a lwlock instead of a spinlock
  *	 so it's less harmful to hold the lock over a WAL write
- *	 (c.f. AdvanceReplicationProgress).
+ *	 (cf. AdvanceReplicationProgress).
  *
  * ---------------------------------------------------------------------------
  */
@@ -147,9 +147,7 @@ typedef struct ReplicationStateOnDisk
 
 typedef struct ReplicationStateCtl
 {
-	/* Tranche to use for per-origin LWLocks */
 	int			tranche_id;
-	/* Array of length max_replication_slots */
 	ReplicationState states[FLEXIBLE_ARRAY_MEMBER];
 } ReplicationStateCtl;
 
@@ -166,10 +164,6 @@ TimestampTz replorigin_session_origin_timestamp = 0;
  * max_replication_slots?
  */
 static ReplicationState *replication_states;
-
-/*
- * Actual shared memory block (replication_states[] is now part of this).
- */
 static ReplicationStateCtl *replication_states_ctl;
 
 /*
@@ -347,7 +341,7 @@ replorigin_drop(RepOriginId roident, bool nowait)
 
 	/*
 	 * To interlock against concurrent drops, we hold ExclusiveLock on
-	 * pg_replication_origin throughout this funcion.
+	 * pg_replication_origin throughout this function.
 	 */
 	rel = heap_open(ReplicationOriginRelationId, ExclusiveLock);
 
@@ -485,7 +479,7 @@ ReplicationOriginShmemSize(void)
 	/*
 	 * XXX: max_replication_slots is arguably the wrong thing to use, as here
 	 * we keep the replay state of *remote* transactions. But for now it seems
-	 * sufficient to reuse it, rather than introduce a separate GUC.
+	 * sufficient to reuse it, lest we introduce a separate GUC.
 	 */
 	if (max_replication_slots == 0)
 		return size;
@@ -515,10 +509,10 @@ ReplicationOriginShmemInit(void)
 	{
 		int			i;
 
-		MemSet(replication_states_ctl, 0, ReplicationOriginShmemSize());
-
 		replication_states_ctl->tranche_id = LWTRANCHE_REPLICATION_ORIGIN;
 
+		MemSet(replication_states, 0, ReplicationOriginShmemSize());
+
 		for (i = 0; i < max_replication_slots; i++)
 		{
 			LWLockInitialize(&replication_states[i].lock,
@@ -573,9 +567,8 @@ CheckPointReplicationOrigin(void)
 	 * no other backend can perform this at the same time, we're protected by
 	 * CheckpointLock.
 	 */
-	tmpfd = OpenTransientFile((char *) tmppath,
-							  O_CREAT | O_EXCL | O_WRONLY | PG_BINARY,
-							  S_IRUSR | S_IWUSR);
+	tmpfd = OpenTransientFile(tmppath,
+							  O_CREAT | O_EXCL | O_WRONLY | PG_BINARY);
 	if (tmpfd < 0)
 		ereport(PANIC,
 				(errcode_for_file_access(),
@@ -705,7 +698,7 @@ StartupReplicationOrigin(void)
 
 	elog(DEBUG2, "starting up replication origin progress state");
 
-	fd = OpenTransientFile((char *) path, O_RDONLY | PG_BINARY, 0);
+	fd = OpenTransientFile(path, O_RDONLY | PG_BINARY);
 
 	/*
 	 * might have had max_replication_slots == 0 last run, or we just brought
@@ -1098,7 +1091,7 @@ replorigin_session_setup(RepOriginId node)
 		{
 			ereport(ERROR,
 					(errcode(ERRCODE_OBJECT_IN_USE),
-					 errmsg("replication origin with OID %d is already active for PID %d",
+					 errmsg("replication identifier %d is already active for PID %d",
 							curstate->roident, curstate->acquired_by)));
 		}
 
