@@ -29,9 +29,9 @@ PostgresNode - class representing PostgreSQL server instance
   # as well as the psql exit code. Pass some extra psql
   # options. If there's an error from psql raise an exception.
   my ($stdout, $stderr, $timed_out);
-  my $cmdret = $node->psql('postgres', 'SELECT pg_sleep(600)',
+  my $cmdret = $node->psql('postgres', 'SELECT pg_sleep(60)',
 	  stdout => \$stdout, stderr => \$stderr,
-	  timeout => 180, timed_out => \$timed_out,
+	  timeout => 30, timed_out => \$timed_out,
 	  extra_params => ['--single-transaction'],
 	  on_error_die => 1)
   print "Sleep timed out" if $timed_out;
@@ -63,9 +63,6 @@ PostgresNode - class representing PostgreSQL server instance
   # Stop the server
   $node->stop('fast');
 
-  # Find a free, unprivileged TCP port to bind some other service to
-  my $port = get_free_port();
-
 =head1 DESCRIPTION
 
 PostgresNode contains a set of routines able to work on a PostgreSQL node,
@@ -105,13 +102,13 @@ use Scalar::Util qw(blessed);
 
 our @EXPORT = qw(
   get_new_node
-  get_free_port
 );
 
 our ($use_tcp, $test_localhost, $test_pghost, $last_host_assigned,
 	$last_port_assigned, @all_nodes, $died);
 
-# For backward compatibility only.
+# Windows path to virtual file system root
+
 our $vfs_path = '';
 if ($Config{osname} eq 'msys')
 {
@@ -455,17 +452,7 @@ sub init
 	print $conf "log_statement = all\n";
 	print $conf "log_replication_commands = on\n";
 	print $conf "wal_retrieve_retry_interval = '500ms'\n";
-
-	# If a setting tends to affect whether tests pass or fail, print it after
-	# TEMP_CONFIG.  Otherwise, print it before TEMP_CONFIG, thereby permitting
-	# overrides.  Settings that merely improve performance or ease debugging
-	# belong before TEMP_CONFIG.
-	print $conf TestLib::slurp_file($ENV{TEMP_CONFIG})
-	  if defined $ENV{TEMP_CONFIG};
-
-	# XXX Neutralize any stats_temp_directory in TEMP_CONFIG.  Nodes running
-	# concurrently must not share a stats_temp_directory.
-	print $conf "stats_temp_directory = 'pg_stat_tmp'\n";
+	print $conf "port = $port\n";
 
 	if ($params{allows_streaming})
 	{
@@ -477,15 +464,13 @@ sub init
 		{
 			print $conf "wal_level = replica\n";
 		}
-		print $conf "max_wal_senders = 10\n";
-		print $conf "max_replication_slots = 10\n";
+		print $conf "max_wal_senders = 5\n";
+		print $conf "max_replication_slots = 5\n";
+		print $conf "max_wal_size = 128MB\n";
+		print $conf "shared_buffers = 1MB\n";
 		print $conf "wal_log_hints = on\n";
 		print $conf "hot_standby = on\n";
-		# conservative settings to ensure we can run multiple postmasters:
-		print $conf "shared_buffers = 1MB\n";
-		print $conf "max_connections = 20\n";
-		# limit disk space consumption, too:
-		print $conf "max_wal_size = 128MB\n";
+		print $conf "max_connections = 10\n";
 	}
 	else
 	{
@@ -493,7 +478,6 @@ sub init
 		print $conf "max_wal_senders = 0\n";
 	}
 
-	print $conf "port = $port\n";
 	if ($use_tcp)
 	{
 		print $conf "unix_socket_directories = ''\n";
@@ -546,11 +530,8 @@ sub append_conf
 =item $node->backup(backup_name)
 
 Create a hot backup with B<pg_basebackup> in subdirectory B<backup_name> of
-B<< $node->backup_dir >>, including the WAL.
-
-By default, WAL files are fetched at the end of the backup, not streamed.
-You can adjust that and other things by passing an array of additional
-B<pg_basebackup> command line options in the keyword parameter backup_options.
+B<< $node->backup_dir >>, including the WAL. WAL files
+fetched at the end of the backup, not streamed.
 
 You'll have to configure a suitable B<max_wal_senders> on the
 target server since it isn't done by default.
@@ -559,16 +540,13 @@ target server since it isn't done by default.
 
 sub backup
 {
-	my ($self, $backup_name, %params) = @_;
+	my ($self, $backup_name) = @_;
 	my $backup_path = $self->backup_dir . '/' . $backup_name;
 	my $name        = $self->name;
 
 	print "# Taking pg_basebackup $backup_name from node \"$name\"\n";
-	TestLib::system_or_bail(
-		'pg_basebackup', '-D', $backup_path, '-h',
-		$self->host,     '-p', $self->port,  '--checkpoint',
-		'fast',          '--no-sync',
-		@{ $params{backup_options} });
+	TestLib::system_or_bail('pg_basebackup', '-D', $backup_path, '-h',
+		$self->host, '-p', $self->port, '--no-sync');
 	print "# Backup finished\n";
 	return;
 }
@@ -774,11 +752,6 @@ sub start
 	{
 		print "# pg_ctl start failed; logfile:\n";
 		print TestLib::slurp_file($self->logfile);
-
-		# pg_ctl could have timed out, so check to see if there's a pid file;
-		# otherwise our END block will fail to shut down the new postmaster.
-		$self->_update_pid(-1);
-
 		BAIL_OUT("pg_ctl start failed") unless $params{fail_ok};
 		return 0;
 	}
@@ -822,37 +795,20 @@ Note: if the node is already known stopped, this does nothing.
 However, if we think it's running and it's not, it's important for
 this to fail.  Otherwise, tests might fail to detect server crashes.
 
-With optional extra param fail_ok => 1, returns 0 for failure
-instead of bailing out.
-
 =cut
 
 sub stop
 {
-	my ($self, $mode, %params) = @_;
+	my ($self, $mode) = @_;
+	my $port   = $self->port;
 	my $pgdata = $self->data_dir;
 	my $name   = $self->name;
-	my $ret;
 	$mode = 'fast' unless defined $mode;
-	return 1 unless defined $self->{_pid};
-
+	return unless defined $self->{_pid};
 	print "### Stopping node \"$name\" using mode $mode\n";
-	$ret = TestLib::system_log('pg_ctl', '-D', $pgdata,
-		'-m', $mode, 'stop');
-
-	if ($ret != 0)
-	{
-		print "# pg_ctl stop failed: $ret\n";
-
-		# Check to see if we still have a postmaster or not.
-		$self->_update_pid(-1);
-
-		BAIL_OUT("pg_ctl stop failed") unless $params{fail_ok};
-		return 0;
-	}
-
+	TestLib::system_or_bail('pg_ctl', '-D', $pgdata, '-m', $mode, 'stop');
 	$self->_update_pid(0);
-	return 1;
+	return;
 }
 
 =pod
@@ -937,7 +893,7 @@ standby_mode=on
 sub enable_restoring
 {
 	my ($self, $root_node) = @_;
-	my $path = TestLib::perl2host($root_node->archive_dir);
+	my $path = $vfs_path . $root_node->archive_dir;
 	my $name = $self->name;
 
 	print "### Enabling WAL restore for node \"$name\"\n";
@@ -966,7 +922,7 @@ standby_mode = on
 sub enable_archiving
 {
 	my ($self) = @_;
-	my $path   = TestLib::perl2host($self->archive_dir);
+	my $path   = $vfs_path . $self->archive_dir;
 	my $name   = $self->name;
 
 	print "### Enabling WAL archiving for node \"$name\"\n";
@@ -992,10 +948,7 @@ archive_command = '$copy_command'
 	return;
 }
 
-# Internal method to update $self->{_pid}
-# $is_running = 1: pid file should be there
-# $is_running = 0: pid file should NOT be there
-# $is_running = -1: we aren't sure
+# Internal method
 sub _update_pid
 {
 	my ($self, $is_running) = @_;
@@ -1006,22 +959,11 @@ sub _update_pid
 	if (open my $pidfile, '<', $self->data_dir . "/postmaster.pid")
 	{
 		chomp($self->{_pid} = <$pidfile>);
+		print "# Postmaster PID for node \"$name\" is $self->{_pid}\n";
 		close $pidfile;
 
-		# If we aren't sure what to expect, validate the PID using kill().
-		# This protects against stale PID files left by crashed postmasters.
-		if ($is_running == -1 && kill(0, $self->{_pid}) == 0)
-		{
-			print
-			  "# Stale postmaster.pid file for node \"$name\": PID $self->{_pid} no longer exists\n";
-			$self->{_pid} = undef;
-			return;
-		}
-
-		print "# Postmaster PID for node \"$name\" is $self->{_pid}\n";
-
 		# If we found a pidfile when there shouldn't be one, complain.
-		BAIL_OUT("postmaster.pid unexpectedly present") if $is_running == 0;
+		BAIL_OUT("postmaster.pid unexpectedly present") unless $is_running;
 		return;
 	}
 
@@ -1029,7 +971,7 @@ sub _update_pid
 	print "# No postmaster PID for node \"$name\"\n";
 
 	# Complain if we expected to find a pidfile.
-	BAIL_OUT("postmaster.pid unexpectedly not present") if $is_running == 1;
+	BAIL_OUT("postmaster.pid unexpectedly not present") if $is_running;
 	return;
 }
 
@@ -1070,22 +1012,54 @@ sub get_new_node
 	my $class = 'PostgresNode';
 	$class = shift if scalar(@_) % 2 != 1;
 	my ($name, %params) = @_;
+	my $port_is_forced = defined $params{port};
+	my $found          = $port_is_forced;
+	my $port = $port_is_forced ? $params{port} : $last_port_assigned;
 
-	# Select a port.
-	my $port;
-	if (defined $params{port})
+	while ($found == 0)
 	{
-		$port = $params{port};
-	}
-	else
-	{
-		# When selecting a port, we look for an unassigned TCP port number,
-		# even if we intend to use only Unix-domain sockets.  This is clearly
-		# necessary on $use_tcp (Windows) configurations, and it seems like a
-		# good idea on Unixen as well.
-		$port = get_free_port();
+
+		# advance $port, wrapping correctly around range end
+		$port = 49152 if ++$port >= 65536;
+		print "# Checking port $port\n";
+
+		# Check first that candidate port number is not included in
+		# the list of already-registered nodes.
+		$found = 1;
+		foreach my $node (@all_nodes)
+		{
+			$found = 0 if ($node->port == $port);
+		}
+
+		# Check to see if anything else is listening on this TCP port.  This
+		# is *necessary* on $use_tcp (Windows) configurations.  Seek a port
+		# available for all possible listen_addresses values, for own_host
+		# nodes and so the caller can harness this port for the widest range
+		# of purposes.  The 0.0.0.0 test achieves that for post-2006 Cygwin,
+		# which automatically sets SO_EXCLUSIVEADDRUSE.  The same holds for
+		# MSYS (a Cygwin fork).  Testing 0.0.0.0 is insufficient for Windows
+		# native Perl (https://stackoverflow.com/a/14388707), so we also test
+		# individual addresses.
+		#
+		# This seems like a good idea on Unixen as well, even though we don't
+		# ask the postmaster to open a TCP port on Unix.  On Non-Linux,
+		# non-Windows kernels, binding to 127.0.0.1/24 addresses other than
+		# 127.0.0.1 fails with EADDRNOTAVAIL.
+		#
+		# XXX A port available now may become unavailable by the time we start
+		# the postmaster.
+		if ($found == 1)
+		{
+			foreach my $addr (qw(127.0.0.1 0.0.0.0),
+				$use_tcp ? qw(127.0.0.2 127.0.0.3) : ())
+			{
+				can_bind($addr, $port) or $found = 0;
+			}
+		}
 	}
 
+	print "# Found port $port\n";
+
 	# Select a host.
 	my $host = $test_pghost;
 	if ($params{own_host})
@@ -1109,81 +1083,12 @@ sub get_new_node
 	# Add node to list of nodes
 	push(@all_nodes, $node);
 
+	# And update port for next time
+	$port_is_forced or $last_port_assigned = $port;
+
 	return $node;
 }
 
-=pod
-
-=item get_free_port()
-
-Locate an unprivileged (high) TCP port that's not currently bound to
-anything.  This is used by get_new_node, and is also exported for use
-by test cases that need to start other, non-Postgres servers.
-
-Ports assigned to existing PostgresNode objects are automatically
-excluded, even if those servers are not currently running.
-
-XXX A port available now may become unavailable by the time we start
-the desired service.
-
-=cut
-
-sub get_free_port
-{
-	my $found = 0;
-	my $port  = $last_port_assigned;
-
-	while ($found == 0)
-	{
-
-		# advance $port, wrapping correctly around range end
-		$port = 49152 if ++$port >= 65536;
-		print "# Checking port $port\n";
-
-		# Check first that candidate port number is not included in
-		# the list of already-registered nodes.
-		$found = 1;
-		foreach my $node (@all_nodes)
-		{
-			$found = 0 if ($node->port == $port);
-		}
-
-		# Check to see if anything else is listening on this TCP port.
-		# Seek a port available for all possible listen_addresses values,
-		# so callers can harness this port for the widest range of purposes.
-		# The 0.0.0.0 test achieves that for MSYS, which automatically sets
-		# SO_EXCLUSIVEADDRUSE.  Testing 0.0.0.0 is insufficient for Windows
-		# native Perl (https://stackoverflow.com/a/14388707), so we also
-		# have to test individual addresses.  Doing that for 127.0.0/24
-		# addresses other than 127.0.0.1 might fail with EADDRNOTAVAIL on
-		# non-Linux, non-Windows kernels.
-		#
-		# Thus, 0.0.0.0 and individual 127.0.0/24 addresses are tested
-		# only on Windows and only when TCP usage is requested.
-		if ($found == 1)
-		{
-			foreach my $addr (qw(127.0.0.1),
-               ($use_tcp && $TestLib::windows_os)
-               ? qw(127.0.0.2 127.0.0.3 0.0.0.0)
-               : ())
-			{
-				if (!can_bind($addr, $port))
-				{
-					$found = 0;
-					last;
-				}
-			}
-		}
-	}
-
-	print "# Found port $port\n";
-
-	# Update port for next time
-	$last_port_assigned = $port;
-
-	return $port;
-}
-
 # Internal routine to check whether a host:port is available to bind
 sub can_bind
 {
@@ -1203,6 +1108,20 @@ sub can_bind
 	return $ret;
 }
 
+# Retain the errno on die() if set, else assume a generic errno of 1.
+# This will instruct the END handler on how to handle artifacts left
+# behind from tests.
+$SIG{__DIE__} = sub {
+	if ($!)
+	{
+		$died = $!;
+	}
+	else
+	{
+		$died = 1;
+	}
+};
+
 # Automatically shut down any still-running nodes when the test script exits.
 # Note that this just stops the postmasters (in the same order the nodes were
 # created in).  Any temporary directories are deleted, in an unspecified
@@ -1221,7 +1140,8 @@ END
 		next if defined $ENV{'PG_TEST_NOCLEAN'};
 
 		# clean basedir on clean test invocation
-		$node->clean_node if $exit_code == 0 && TestLib::all_tests_passing();
+		$node->clean_node
+		  if TestLib::all_tests_passing() && !defined $died && !$exit_code;
 	}
 
 	$? = $exit_code;
@@ -1293,6 +1213,7 @@ sub safe_psql
 		print "\n#### End standard error\n";
 	}
 
+	$stdout =~ s/\r//g if $TestLib::windows_os;
 	return $stdout;
 }
 
@@ -1363,9 +1284,9 @@ If given, it must be an array reference containing additional parameters to B<ps
 e.g.
 
 	my ($stdout, $stderr, $timed_out);
-	my $cmdret = $node->psql('postgres', 'SELECT pg_sleep(600)',
+	my $cmdret = $node->psql('postgres', 'SELECT pg_sleep(60)',
 		stdout => \$stdout, stderr => \$stderr,
-		timeout => 180, timed_out => \$timed_out,
+		timeout => 30, timed_out => \$timed_out,
 		extra_params => ['--single-transaction'])
 
 will set $cmdret to undef and $timed_out to a true value.
@@ -1468,20 +1389,16 @@ sub psql
 		}
 	};
 
-	# Note: on Windows, IPC::Run seems to convert \r\n to \n in program output
-	# if we're using native Perl, but not if we're using MSys Perl.  So do it
-	# by hand in the latter case, here and elsewhere.
-
 	if (defined $$stdout)
 	{
-		$$stdout =~ s/\r\n/\n/g if $Config{osname} eq 'msys';
 		chomp $$stdout;
+		$$stdout =~ s/\r//g if $TestLib::windows_os;
 	}
 
 	if (defined $$stderr)
 	{
-		$$stderr =~ s/\r\n/\n/g if $Config{osname} eq 'msys';
 		chomp $$stderr;
+		$$stderr =~ s/\r//g if $TestLib::windows_os;
 	}
 
 	# See http://perldoc.perl.org/perlvar.html#%24CHILD_ERROR
@@ -1519,231 +1436,6 @@ sub psql
 
 =pod
 
-=item $node->background_psql($dbname, \$stdin, \$stdout, $timer, %params) => harness
-
-Invoke B<psql> on B<$dbname> and return an IPC::Run harness object, which the
-caller may use to send input to B<psql>.  The process's stdin is sourced from
-the $stdin scalar reference, and its stdout and stderr go to the $stdout
-scalar reference.  This allows the caller to act on other parts of the system
-while idling this backend.
-
-The specified timer object is attached to the harness, as well.  It's caller's
-responsibility to select the timeout length, and to restart the timer after
-each command if the timeout is per-command.
-
-psql is invoked in tuples-only unaligned mode with reading of B<.psqlrc>
-disabled.  That may be overridden by passing extra psql parameters.
-
-Dies on failure to invoke psql, or if psql fails to connect.  Errors occurring
-later are the caller's problem.  psql runs with on_error_stop by default so
-that it will stop running sql and return 3 if passed SQL results in an error.
-
-Be sure to "finish" the harness when done with it.
-
-=over
-
-=item on_error_stop => 1
-
-By default, the B<psql> method invokes the B<psql> program with ON_ERROR_STOP=1
-set, so SQL execution is stopped at the first error and exit code 3 is
-returned.  Set B<on_error_stop> to 0 to ignore errors instead.
-
-=item replication => B<value>
-
-If set, add B<replication=value> to the conninfo string.
-Passing the literal value C<database> results in a logical replication
-connection.
-
-=item extra_params => ['--single-transaction']
-
-If given, it must be an array reference containing additional parameters to B<psql>.
-
-=back
-
-=cut
-
-sub background_psql
-{
-	my ($self, $dbname, $stdin, $stdout, $timer, %params) = @_;
-
-	local $ENV{PGHOST} = $self->host;
-	local $ENV{PGPORT} = $self->port;
-
-	my $replication = $params{replication};
-
-	my @psql_params = (
-		'psql',
-		'-XAtq',
-		'-d',
-		$self->connstr($dbname)
-		  . (defined $replication ? " replication=$replication" : ""),
-		'-f',
-		'-');
-
-	$params{on_error_stop} = 1 unless defined $params{on_error_stop};
-
-	push @psql_params, '-v', 'ON_ERROR_STOP=1' if $params{on_error_stop};
-	push @psql_params, @{ $params{extra_params} }
-	  if defined $params{extra_params};
-
-	# Ensure there is no data waiting to be sent:
-	$$stdin = "" if ref($stdin);
-	# IPC::Run would otherwise append to existing contents:
-	$$stdout = "" if ref($stdout);
-
-	my $harness = IPC::Run::start \@psql_params,
-	  '<', $stdin, '>', $stdout, $timer;
-
-	# Request some output, and pump until we see it.  This means that psql
-	# connection failures are caught here, relieving callers of the need to
-	# handle those.  (Right now, we have no particularly good handling for
-	# errors anyway, but that might be added later.)
-	my $banner = "background_psql: ready";
-	$$stdin = "\\echo $banner\n";
-	pump $harness until $$stdout =~ /$banner/ || $timer->is_expired;
-
-	die "psql startup timed out" if $timer->is_expired;
-
-	return $harness;
-}
-
-# Common sub of pgbench-invoking interfaces.  Makes any requested script files
-# and returns pgbench command-line options causing use of those files.
-sub _pgbench_make_files
-{
-	my ($self, $files) = @_;
-	my @file_opts;
-
-	if (defined $files)
-	{
-
-		# note: files are ordered for determinism
-		for my $fn (sort keys %$files)
-		{
-			my $filename = $self->basedir . '/' . $fn;
-			push @file_opts, '-f', $filename;
-
-			# cleanup file weight
-			$filename =~ s/\@\d+$//;
-
-			#push @filenames, $filename;
-			# filenames are expected to be unique on a test
-			if (-e $filename)
-			{
-				ok(0, "$filename must not already exist");
-				unlink $filename or die "cannot unlink $filename: $!";
-			}
-			TestLib::append_to_file($filename, $$files{$fn});
-		}
-	}
-
-	return @file_opts;
-}
-
-=pod
-
-=item $node->pgbench($opts, $stat, $out, $err, $name, $files, @args)
-
-Invoke B<pgbench>, with parameters and files.
-
-=over
-
-=item $opts
-
-Options as a string to be split on spaces.
-
-=item $stat
-
-Expected exit status.
-
-=item $out
-
-Reference to a regexp list that must match stdout.
-
-=item $err
-
-Reference to a regexp list that must match stderr.
-
-=item $name
-
-Name of test for error messages.
-
-=item $files
-
-Reference to filename/contents dictionary.
-
-=item @args
-
-Further raw options or arguments.
-
-=back
-
-=cut
-
-sub pgbench
-{
-	local $Test::Builder::Level = $Test::Builder::Level + 1;
-
-	my ($self, $opts, $stat, $out, $err, $name, $files, @args) = @_;
-	my @cmd = (
-		'pgbench',
-		split(/\s+/, $opts),
-		$self->_pgbench_make_files($files), @args);
-
-	$self->command_checks_all(\@cmd, $stat, $out, $err, $name);
-}
-
-=pod
-
-=item $node->background_pgbench($opts, $files, \$stdout, $timer) => harness
-
-Invoke B<pgbench> and return an IPC::Run harness object.  The process's stdin
-is empty, and its stdout and stderr go to the $stdout scalar reference.  This
-allows the caller to act on other parts of the system while B<pgbench> is
-running.  Errors from B<pgbench> are the caller's problem.
-
-The specified timer object is attached to the harness, as well.  It's caller's
-responsibility to select the timeout length, and to restart the timer after
-each command if the timeout is per-command.
-
-Be sure to "finish" the harness when done with it.
-
-=over
-
-=item $opts
-
-Options as a string to be split on spaces.
-
-=item $files
-
-Reference to filename/contents dictionary.
-
-=back
-
-=cut
-
-sub background_pgbench
-{
-	my ($self, $opts, $files, $stdout, $timer) = @_;
-
-	my @cmd =
-	  ('pgbench', split(/\s+/, $opts), $self->_pgbench_make_files($files));
-
-	local $ENV{PGHOST} = $self->host;
-	local $ENV{PGPORT} = $self->port;
-
-	my $stdin = "";
-	# IPC::Run would otherwise append to existing contents:
-	$$stdout = "" if ref($stdout);
-
-	my $harness = IPC::Run::start \@cmd, '<', \$stdin, '>', $stdout, '2>&1',
-	  $timer;
-
-	return $harness;
-}
-
-=pod
-
 =item $node->poll_query_until($dbname, $query [, $expected ])
 
 Run B<$query> repeatedly, until it returns the B<$expected> result
@@ -1760,22 +1452,19 @@ sub poll_query_until
 
 	$expected = 't' unless defined($expected);    # default value
 
-	my $cmd = [ 'psql', '-XAt', '-d', $self->connstr($dbname) ];
+	my $cmd = [ 'psql', '-XAt', '-c', $query, '-d', $self->connstr($dbname) ];
 	my ($stdout, $stderr);
 	my $max_attempts = 180 * 10;
 	my $attempts     = 0;
 
 	while ($attempts < $max_attempts)
 	{
-		my $result = IPC::Run::run $cmd, '<', \$query,
-		  '>', \$stdout, '2>', \$stderr;
+		my $result = IPC::Run::run $cmd, '>', \$stdout, '2>', \$stderr;
 
-		$stdout =~ s/\r\n/\n/g if $Config{osname} eq 'msys';
 		chomp($stdout);
-		$stderr =~ s/\r\n/\n/g if $Config{osname} eq 'msys';
-		chomp($stderr);
+		$stdout =~ s/\r//g if $TestLib::windows_os;
 
-		if ($stdout eq $expected && $stderr eq '')
+		if ($stdout eq $expected)
 		{
 			return 1;
 		}
@@ -1788,6 +1477,8 @@ sub poll_query_until
 
 	# The query result didn't change in 180 seconds. Give up. Print the
 	# output from the last attempt, hopefully that's useful for debugging.
+	chomp($stderr);
+	$stderr =~ s/\r//g if $TestLib::windows_os;
 	diag qq(poll_query_until timed out executing this query:
 $query
 expecting this output:
@@ -1884,6 +1575,9 @@ sub command_checks_all
 Run a command on the node, then verify that $expected_sql appears in the
 server log file.
 
+Reads the whole log file so be careful when working with large log outputs.
+The log file is truncated prior to running the command, however.
+
 =cut
 
 sub issues_sql_like
@@ -1893,11 +1587,10 @@ sub issues_sql_like
 	local $ENV{PGHOST} = $self->host;
 	local $ENV{PGPORT} = $self->port;
 
-	my $log_location = -s $self->logfile;
-
+	truncate $self->logfile, 0;
 	my $result = TestLib::run_log($cmd);
 	ok($result, "@$cmd exit code 0");
-	my $log = TestLib::slurp_file($self->logfile, $log_location);
+	my $log = TestLib::slurp_file($self->logfile);
 	like($log, $expected_sql, "$test_name: SQL found in server log");
 	return;
 }
@@ -2219,8 +1912,8 @@ sub pg_recvlogical_upto
 		}
 	};
 
-	$stdout =~ s/\r\n/\n/g if $Config{osname} eq 'msys';
-	$stderr =~ s/\r\n/\n/g if $Config{osname} eq 'msys';
+	$stdout =~ s/\r//g if $TestLib::windows_os;
+	$stderr =~ s/\r//g if $TestLib::windows_os;
 
 	if (wantarray)
 	{
